<?php
/**
 * dejure.org Vernetzungsfunktion Plugin
 *
 * @license    GPL v2 (http://www.gnu.org/licenses/gpl.html)
 * @author     dejure.org <info@dejure.org>
 * @link       http://dejure.org/vernetzung.html
 * @version    1.53
 */

# Von Ihnen aenderbare Variablen
/****************************************/

define('DJO_ANBIETERKENNUNG', '');		// Hier eine feste Bezeichnung für Ihren Dienst zur Kommunikation mit dejure.org festlegen (Buchstaben und Zahlen erlaubt, mindestens 5 Stellen).
										// Wenn Sie diese leer lassen, wird automatisch die Webadresse und der Titel des Wikis verwendet.
define('DJO_KONTAKTEMAIL',    '');		// Wird im Rahmen der Serverkommunikation jeweils an dejure.org übertragen und dient im Übrigen ausschließlich der Kommunikation bei technischen Problemen.
define('DJO_TARGET',          '');		// Ziel der Links. Zum Beispiel für Öffnen in neuem Fenster: "_blank"
define('DJO_CSSCLASS',        '');		// CSS-Klasse für die dejure.org-Links. Falls Sie diese optisch hervorheben möchten.
define('DJO_FORMAT',          'weit');	// "weit"   = möglichst lange Verlinkung (Bsp.: <a>§ 242 BGB</a>; <a>§§ 278</a>, <a>254 BGB</a>)
										// "schmal" = nur die Nummern der Vorschriften verlinken (Bsp.: § <a>242</a> BGB; §§ <a>278</a>, <a>254</a> BGB)
define('DJO_BUZER',           1);		// (1|0) Zu buzer.de verlinken für Gesetze, die bei dejure.org nicht vorhanden sind
define('DJO_OHNEHTAGS',       0);		// Keine Vernetzung innerhalb von Überschriften (h1 bis h9)
define('DJO_TIMEOUT',         2);		// Maximale Wartezeit für Serverkommunikation in Sekunden. (empfohlen: 2)
										// Die Kommunikation dauert nur bei sehr grossen Seiten länger als 1 Sekunde. Eine leichte Verzögerung aufgrund der Serverkommunikation tritt wegen des eingebauten Caches nur sporadisch auf.

/****************************************/


#######################################################
# Ab hier bitte nichts aendern
#######################################################

define('DJO_VERSION', '1.53');

if (!defined('DOKU_INC'))
	die();

if (!defined('DOKU_PLUGIN'))
	define('DOKU_PLUGIN', DOKU_INC . 'lib/plugins/');

require_once (DOKU_PLUGIN . 'action.php');

class action_plugin_dejureorgvernetzungdokuwiki extends DokuWiki_Action_Plugin {

	public function getInfo() {
		return array(
				'author' => 'dejure.org',
				'email'  => 'info@dejure.org',
				'date'   => '2020-06-18',
				'name'   => 'dejure.org Vernetzungsfunktion',
				'desc'   => 'Diese Erweiterung verlinkt automatisch zitierte Gesetze und Rechtsprechung (Aktenzeichen und Fundstellen) mit den Inhalten von dejure.org. Sie können die Vernetzungsfunktion anpassen, indem Sie die Datei "dokuwiki/lib/plugins/dejureorgvernetzungdokuwiki/action.php" bearbeiten."',
				'url'    => 'https://dejure.org/vernetzung.html',
				);
	}

	function register(Doku_Event_Handler $controller) {
		$controller->register_hook('RENDERER_CONTENT_POSTPROCESS', 'BEFORE', $this, '_vernetzen');
	}

	public function _vernetzen(& $event, $param) {
		if ($event->data[0] != 'xhtml') {   // wenn kein xhtml, dann nicht vernetzen
			return;
		}

		$Anbieterkennung = DJO_KONTAKTEMAIL.'|'.DJO_ANBIETERKENNUNG;
		if (DJO_KONTAKTEMAIL == '' || DJO_ANBIETERKENNUNG == '') {
			$Anbieterkennung = $_SERVER['HTTP_HOST'].'|'.$conf['title'];
		}
		$parameter = array(
			'Anbieterkennung'		=> urlencode($Anbieterkennung),
			'format'				=> DJO_FORMAT,
			'target'				=> DJO_TARGET,
			'class'					=> DJO_CSSCLASS,
			'version'				=> 'dokuwiki-'.DJO_VERSION,
			'buzer'					=> DJO_BUZER,
			'ohnehtags'				=> DJO_OHNEHTAGS,
			'zeitlimit_in_sekunden'	=> DJO_TIMEOUT,
			'Schema'				=> 'https');

		$html = &$event->data[1];
		if (preg_match("/§|&sect;|&#167;|Artikel|Art\.|[\/\.] ?[0-9][0-9](?![0-9\/])| [0-9][0-9]?[\/\.] ?[0-9][0-9](?![0-9\.])| [A-Z]+ [0-9][0-9]\. ?[0-9]|[0-9][0-9],/", $html) || preg_match("/<!-- *ohnedjo *-->/", $html)) {
			$html = $this->vernetzen_ueber_dejure_org($html, $parameter);
		}
    }

	function vernetzen_ueber_dejure_org($ausgangstext, $parameter = array()) {
		// Moegliche Parameter: Anbieterkennung / Dokumentkennung / target / class / AktenzeichenIgnorieren / zeitlimit_in_sekunden
		global $djo_vernetzung_erfolgt;
		$djo_vernetzung_erfolgt = 0;
		$uebergabe = 'Originaltext='.urlencode($ausgangstext);
		foreach ($parameter as $option => $wert) {
			if ($option == 'zeitlimit_in_sekunden') {
				$zeitlimit_in_sekunden = $wert;
			}
			else {
				$uebergabe .= '&'.urlencode($option).'='.urlencode($wert);
			}
		}
		if (!isset($zeitlimit_in_sekunden) || empty($zeitlimit_in_sekunden)) {
			$zeitlimit_in_sekunden = 2;
		}
		$anbieter = DJO_ANBIETERKENNUNG;
		if (empty($anbieter)) {
			$anbieter = $_SERVER['SERVER_NAME'];
		}
		$header = 'POST /dienste/vernetzung/vernetzen HTTP/1.0'."\r\n";
		$header.= 'User-Agent: '.$anbieter.' (Dokuwiki-Vernetzung '.DJO_VERSION.')'."\r\n";
		$header.= 'Content-type: application/x-www-form-urlencoded'."\r\n";
		$header.= 'Content-length: '.strlen($uebergabe)."\r\n";
		$header.= 'Host: rechtsnetz.dejure.org'."\r\n";
		$header.= 'Connection: close'."\r\n";
		$header.= "\r\n";

		$fp = @fsockopen('tsl://rechtsnetz.dejure.org', 443, $errno, $errstr, $zeitlimit_in_sekunden);

		if (!$fp) {
			$fp = fsockopen('rechtsnetz.dejure.org', 80, $errno, $errstr, $zeitlimit_in_sekunden);
		}
		if (!$fp) { // Verbindung gescheitert
			return $ausgangstext;
		} else {
			stream_set_timeout($fp, $zeitlimit_in_sekunden, 0); // Socket nach $zeitlimit_in_sekunden Sekunden auf jeden Fall wieder frei geben
			stream_set_blocking($fp, true);
			fputs($fp, $header.$uebergabe);
			$timeOutSock = false;
			$eofSock = false;
			$rueckgabe = '';
			while (!$eofSock && !$timeOutSock) {
				$rueckgabe.= fgets($fp, 1024); //
				$stSock = stream_get_meta_data($fp);
				$eofSock = $stSock['eof'];
				$timeOutSock = $stSock['timed_out'];
			}
			fclose($fp);
			if (!preg_match("/^(.*?)\r?\n\r?\n\r?\n?(.*)/s",$rueckgabe, $rueckgabeARR)) {
				return $ausgangstext; // Zeitueberschreitung oder Verbindungsproblem
			} elseif (strpos($rueckgabeARR[1],'200 OK') === false) {
				return $ausgangstext; // sonstiges Serverproblem
			} else {
				$rueckgabe = $rueckgabeARR[2];
				if (strlen($rueckgabe) < strlen($ausgangstext)) {
					return $ausgangstext;
				}
				$djo_vernetzung_erfolgt = 1;
				return $rueckgabe;
			}
		}
	}

}
